<?php

namespace Drupal\flysystem_drupal_cache;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use League\Flysystem\Adapter\AbstractAdapter;
use League\Flysystem\Adapter\Polyfill\StreamedTrait;
use League\Flysystem\AdapterInterface;
use League\Flysystem\Config;
use League\Flysystem\FileNotFoundException;
use League\Flysystem\Util;
use Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface;

/**
 * Class DrupalCacheAdapter
 *
 * @package Drupal\flysystem_drupal_cache
 */
class DrupalCacheAdapter extends AbstractAdapter {

  use StreamedTrait;

  /**
   * Cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * Cache tags invalidator.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * Mime type guesser.
   *
   * @var \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface
   */
  protected $mimeTypeGuesser;

  /**
   * Default cache tags.
   * @var array
   */
  protected $tags;

  /**
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   Cache backend.
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cacheTagsInvalidator
   *   Cache tags invalidator.
   * @param \Symfony\Component\HttpFoundation\File\MimeType\MimeTypeGuesserInterface $mimeTypeGuesser
   *   Mime type guesser.
   * @param array $tags
   *   Default cache tags.
   */
  public function __construct(CacheBackendInterface $cache, CacheTagsInvalidatorInterface $cacheTagsInvalidator, MimeTypeGuesserInterface $mimeTypeGuesser, array $tags) {
    $this->cacheBackend = $cache;
    $this->cacheTagsInvalidator = $cacheTagsInvalidator;
    $this->mimeTypeGuesser = $mimeTypeGuesser;
    $this->tags = $tags;
  }

  /**
   * Returns the StorageClient.
   *
   * @return \Drupal\Core\Cache\CacheBackendInterface
   */
  public function getCacheBackend() {
    return $this->cacheBackend;
  }

  /**
   * Get the default tags.
   *
   * @return array
   */
  public function getTags() {
    return $this->tags;
  }

  /**
   * Get the cache tags for the file.
   *
   * @param string $path
   *
   * @return array
   */
  private function getPathTags($path) {
    $tags = array_merge($this->getTags(), [$path]);

    $pathSegments = explode('/', $path);
    $previous = '';
    foreach ($pathSegments as $pathSegment) {
      $tags[] = $previous . $pathSegment;
      $previous = $pathSegment . ':';
    }

    return $tags;
  }

  /**
   * Get the cache tags invalidator.
   *
   * @return \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  private function getCacheTagsInvalidator() {
    return $this->cacheTagsInvalidator;
  }

  /**
   * @inheritDoc
   */
  public function write($path, $contents, Config $config) {
    $data = [
      'mimetype' => $this->mimeTypeGuesser->guess($path),
      'contents' => base64_encode($contents)
    ];
    $this->getCacheBackend()->set($path, $data, Cache::PERMANENT, $this->getPathTags($path));
  }

  /**
   * @inheritDoc
   */
  public function update($path, $contents, Config $config) {
    $this->write($path, $contents, $config);
  }

  /**
   * @inheritDoc
   */
  public function rename($path, $newpath) {
    $this->copy($path, $newpath);
    $this->delete($path);
  }

  /**
   * @inheritDoc
   */
  public function copy($path, $newpath) {
    $cacheItem = $this->getCacheBackend()->get($path);
    $this->getCacheBackend()->set($newpath, $cacheItem->data, $this->getPathTags($path));
  }

  /**
   * @inheritDoc
   */
  public function delete($path) {
    $this->getCacheBackend()->delete($path);
    $this->getCacheTagsInvalidator()->invalidateTags([$path]);
  }

  /**
   * @inheritDoc
   */
  public function deleteDir($dirname) {
    $this->getCacheTagsInvalidator()->invalidateTags([$dirname]);
  }

  /**
   * @inheritDoc
   */
  public function createDir($dirname, Config $config) {
    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function setVisibility($path, $visibility) {
    return FALSE;
  }

  /**
   * @inheritDoc
   */
  public function has($path) {
    $cacheItem = $this->getCacheBackend()->get($path);
    return empty(pathinfo($path, PATHINFO_EXTENSION)) || is_object($cacheItem);
  }

  /**
   * @inheritDoc
   */
  public function read($path) {
    $cacheItem = $this->getCacheBackend()->get($path);
    if (!is_object($cacheItem)) {
      return false;
    }

    $data = $cacheItem->data;
    $data['contents'] = base64_decode($data['contents']);

    return $data;
  }

  /**
   * @inheritDoc
   */
  public function listContents($directory = '', $recursive = FALSE) {
    return [];
  }

  /**
   * @inheritDoc
   *
   * @throws \League\Flysystem\FileNotFoundException
   */
  public function getMetadata($path) {
    if (!$this->has($path)) {
      throw new FileNotFoundException("File not found at path: {$path}");
    }

    $cacheItem = $this->getCacheBackend()->get($path);
    $contents = base64_decode($cacheItem->data['contents']);

    $type = 'file';
    if (empty(strlen($contents)) || empty(pathinfo($path, PATHINFO_EXTENSION))) {
      $type = 'dir';
    }

    return [
      'type' => $type,
      'size' => Util::contentSize($contents),
      'mimetype' => $cacheItem->data['mimetype'],
      'timestamp' => $cacheItem->created,
      'visibility' => AdapterInterface::VISIBILITY_PUBLIC,
    ];
  }

  /**
   * @inheritDoc
   *
   * @throws \League\Flysystem\FileNotFoundException
   */
  public function getSize($path) {
    $this->getMetadata($path)['size'];
  }

  /**
   * @inheritDoc
   *
   * @throws \League\Flysystem\FileNotFoundException
   */
  public function getMimetype($path) {
    $this->getMetadata($path)['mimetype'];
  }

  /**
   * @inheritDoc
   *
   * @throws \League\Flysystem\FileNotFoundException
   */
  public function getTimestamp($path) {
    $this->getMetadata($path)['timestamp'];
  }

  /**
   * @inheritDoc
   */
  public function getVisibility($path) {
    return FALSE;
  }
}
